///	FactoryObject.h - Base class of all objects constructed from the factory
///					  (i.e. any component you want the user to be able to
///					  place, customise etc.).
///	--------------------------------------------------------------------------
///	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
///	Copyright (C) 2004-2005  Niall Moody
///	
///	This program is free software; you can redistribute it and/or modify
///	it under the terms of the GNU General Public License as published by
///	the Free Software Foundation; either version 2 of the License, or
///	(at your option) any later version.
///	
///	This program is distributed in the hope that it will be useful,
///	but WITHOUT ANY WARRANTY; without even the implied warranty of
///	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
///	GNU General Public License for more details.
///	
///	You should have received a copy of the GNU General Public License
///	along with this program; if not, write to the Free Software
///	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
///	
///	--------------------------------------------------------------------------

#ifndef FACTORYOBJECT_H_
#define FACTORYOBJECT_H_

class ComponentFactory; //to tidy up dependencies

///	JUCE throws an assert if you feed it relative file paths, so this converts them.
String convertRelativeFilePath(String path);

///	A base class used to represent a component which is instantiated from an xml file, through ComponentFactory.
/*!
	To create a scriptable GUI, you should make a subclass of this class for
	every component on the GUI that you want the user to be able to
	script/alter.

	In the constructor, you first call ComponentFactory::registerObject()
	with a string containing the text you want the component to be named as
	in the xml file.  This lets the ComponentFactory know that this component
	exists, and should have an entry in the xml file.

	For every attribute of the component that you want the user to have
	control of, you create a static method to set it, then (again in the
	constructor) you call ComponentFactory::registerStringAttribute(),
	ComponentFactory::registerDoubleAttribute(),
	ComponentFactory::registerIntAttribute(), or
	ComponentFactory::registerBoolAttribute(), depending on the type of
	the attribute in question.  This registers the associated method with the
	ComponentFactory, so that when it finds the associated attribute in the
	xml file, it knows what to do to set it correctly.  Again, provide the
	function with a string containing the name you want the attribute to have
	in the xml file.

	For the whole thing to work together, you first construct a
	ComponentFactory, then construct all the FactoryObject subclasses,
	passing them a pointer to your ComponentFactory, so they can register
	themselves with the factory.  You then call
	ComponentFactory::createComponents(), which first parses the xml file
	and sets the FactoryObjects' attributes accordingly, then it constructs
	the actual objects for your GUI, by calling the create() methods for all
	the FactoryObject subclasses in turn.

	\sa ComponentFactory

	Note that this approach assumes that there will only ever be one instance
	of any FactoryObject subclass in existence at any one time (the static
	member functions).  It would be useful in the future to somehow force the
	programmer to stick to this approach (singletons, maybe?).
 */
class FactoryObject
{
  public:
	///	Constructor.
	/*!
		\param factory A pointer to the ComponentFactory which will take
		care of this FactoryObject.
		\param windowComponent This is a pointer to a Component pointer.  The
		idea is that you pass the address of the Component pointer you have
		in your main window class to the FactoryObject, so that it can set
		the address the pointer points to.  This is because the Component
		will actually be created in the create() method of the FactoryObject,
		and it will have to inform the main window class that the Component
		in question has been created.
	 */
	FactoryObject(ComponentFactory *factory, Component **windowComponent);
	///	Destructor.
	virtual ~FactoryObject() {};

	/// Called by ComponentFactory to create the actual Component (& stored int windowComponent, so the ScriptedMainWindow can use it).
	/*!
		(all the necessary attributes should have been set when
		ComponentFactory parsed the xml file - i.e. before it called this
		method).
	 */
	virtual void create() = 0;

	///	Used by ComponentFactory::resizeComponents() to resize the Component.
	Component *getComponent() {return *windowComponent;};


	/// Position & sizing attributes are common to all FactoryObjects.
	virtual void setX(int val) {x = val;};
	/// Position & sizing attributes are common to all FactoryObjects.
	virtual void setY(int val) {y = val;};
	/// Position & sizing attributes are common to all FactoryObjects.
	virtual void setXRel(double val) {xRel = (float)val; x = -1;};
	/// Position & sizing attributes are common to all FactoryObjects.
	virtual void setYRel(double val) {yRel = (float)val; y = -1;};
	/// Position & sizing attributes are common to all FactoryObjects.
	virtual void setWidth(int val) {width = val;};
	/// Position & sizing attributes are common to all FactoryObjects.
	virtual void setHeight(int val) {height = val;};
	/// Position & sizing attributes are common to all FactoryObjects.
	virtual void setWidthRel(double val) {widthRel = (float)val; width = -1;};
	/// Position & sizing attributes are common to all FactoryObjects.
	virtual void setHeightRel(double val) {heightRel = (float)val; height = -1;};
	/// Used to hide/show Components (Components are shown by default).
	virtual void setEnabled(bool val) {enabled = val;};

	/// This is common to all FactoryObjects - if it returns -1 it should be ignored.  Called from ComponentFactory::resizeComponents().
	virtual int getX() {return x;};
	/// This is common to all FactoryObjects - if it returns -1 it should be ignored.  Called from ComponentFactory::resizeComponents().
	virtual int getY() {return y;};
	/// (relative = percentage of main window) This is common to all FactoryObjects - if it returns -1 it should be ignored.  Called from ComponentFactory::resizeComponents().
	virtual float getXRel() {return xRel;};
	/// (relative = percentage of main window) This is common to all FactoryObjects - if it returns -1 it should be ignored.  Called from ComponentFactory::resizeComponents().
	virtual float getYRel() {return yRel;};

	/// This is common to all FactoryObjects - if it returns -1 it should be ignored.  Called from ComponentFactory::resizeComponents().
	virtual int getWidth() {return width;};
	/// This is common to all FactoryObjects - if it returns -1 it should be ignored.  Called from ComponentFactory::resizeComponents().
	virtual int getHeight() {return height;};
	/// (relative = percentage of main window) This is common to all FactoryObjects - if it returns -1 it should be ignored.  Called from ComponentFactory::resizeComponents().
	virtual float getWidthRel() {return widthRel;};
	/// (relative = percentage of main window) This is common to all FactoryObjects - if it returns -1 it should be ignored.  Called from ComponentFactory::resizeComponents().
	virtual float getHeightRel() {return heightRel;};

	///	This is common to all FactoryObjects, and is used to show/hide components on the main window.
	virtual bool getEnabled() {return enabled;};

  protected:
	///	There's only ever going to be one factory in existence, so this is ok...
	static ComponentFactory *factory;
	///	A pointer to a Component pointer, where the actual Component this FactoryObject handles exists.
	Component **windowComponent;

	int x;
	int y;
	float xRel;
	float yRel;
	int width;
	int height;
	float widthRel;
	float heightRel;
	bool enabled;
};

#endif
