//	NPBMappingsDialog.h - Dialog window which holds all the mappings
//						  currently set up.
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#ifndef NPBMAPPINGSDIALOG_H_
#define NPBMAPPINGSDIALOG_H_

#ifndef NBPPLUGIN_H_
#include "NPBPlugin.h"
#endif

#include "NPBMapping.h"
#include "NiallsLabel.h"

#include <juce.h>

///	DialogWindow to allow the user to set up mappings between a joystick and a plugin.
class NPBMappingsDialog : public DialogWindow
{
  public:
	///	Constructor.
	/*!
		\param parent Points to the plugin whose mappings this dialog sets.
		\param array A pointer to an array of the current mappings for this
		plugin.
		\param col The background colour for the dialog.
	 */
	NPBMappingsDialog(NPBPlugin *parent, OwnedArray<NPBMapping> *array, Colour col);
	///	Destructor.
	~NPBMappingsDialog() {};
};

///	The contents of NPBMappingsDialog.
/*!
	Derives from SimpleListBoxModel because we use a list box to display the
	mappings.
 */
class NPBMappingsContent : public Component,
								  ActionListener,
								  SimpleListBoxModel
{
  public:
	///	Constructor.
	/*!
		\param parent Points to the plugin whose mappings this dialog sets.
		\param array A pointer to an array of the current mappings for this
		plugin.
		\param col The background colour for the dialog.
	 */
	NPBMappingsContent(NPBPlugin *parent, OwnedArray<NPBMapping> *array, Colour col);
	///	Destructor.
	~NPBMappingsContent();

	///	Paints the background the appropriate colour.
	/*!
		(is this necessary?)
	 */
	void paint(Graphics& g);
	///	So we are notified when the user presses the various text buttons.
	void actionListenerCallback(const String& m);

	///	To place the various Components in the correct positions.
	void resized();

	///	One of the methods that must be implemented if you inherit from SimpleListBoxModel.
	int getNumRows();
	///	Draws the contents of the list box.
	void paintListBoxItem(int rowNumber, Graphics &g, int width, int height, bool rowIsSelected);

	///	Called from NPBMappingAdd & NPBMappingAddMIDI to add a new NPBMapping to our temporary array.
	void addMapping(NPBMapping *map);
  private:
	NPBPlugin *Parent;

	SimpleListBox *Mappings;
	TextButton *AddParameter;
	TextButton *AddMIDI;
	TextButton *Remove;
	TextButton *OK;
	TextButton *Cancel;
	ToggleButton *acceptMIDI;

	///	Temporary array of NPBMappings.
	/*!
		We keep a temporary array of mappings, because we don't want to commit
		changes to the plugin's array unless the user presses the OK button.

		This array is filled with the contents of the plugin's array in the
		constructor.
	 */
	OwnedArray<NPBMapping> MappingsArray;

	///	The colour to draw the background.
	Colour backColour;
};

//----------------------------------------------------------------
///	Simple dialog allowing the user to add a VST parameter mapping to NPBMappingsContent's temporary array.
/*!
	\todo Label the various Components.
	\sa NPBMappingAddMIDI
 */
class NPBMappingAdd : public DialogWindow
{
  public:
	///	Constructor.
	/*!
		\param parent The NPBMappingsContent which can be considered the
		parent of this object.
		\param plugin We need to interrogate the plugin to determine the
		names of all it's parameters.
		\param The colour of the background.
	 */
	NPBMappingAdd(NPBMappingsContent *parent, NPBPlugin *plugin, Colour col);
	///	Destructor.
	~NPBMappingAdd() {};
};

///	The contents of NPBMappingAdd.
class NPBAddContent : public Component,
							 ActionListener
{
  public:
	///	Constructor.
	/*!
		\param parent The NPBMappingsContent which can be considered the
		parent of this object.
		\param plugin We need to interrogate the plugin to determine the
		names of all it's parameters.
		\param The colour of the background.
	 */
	NPBAddContent(NPBMappingsContent *parent, NPBPlugin *plugin, Colour col);
	///	Destructor.
	~NPBAddContent() {};

	///	Fills in the background with colour backColour.
	void paint(Graphics& g);
	///	This is used to draw a frame around the NiallsLabel Component.
	void paintOverChildren(Graphics& g);
	///	So we're notified when the user presses OK or Cancel.
	void actionListenerCallback(const String& m);

	///	To make sure the Components are all placed in the correct positions.
	void resized();
  private:
	//void setupComboBox(ComboBox *cb, int index);

	ComboBox *JoyNum;
	NiallsLabel *JoyControl;
	TextEditor *MinVal;
	TextEditor *MaxVal;
	ComboBox *PluginParameter;

	TextButton *OK;
	TextButton *Cancel;

	NPBMappingsContent *Parent;
	NPBPlugin *Plugin;

	///	Colour to paint the background.
	Colour backColour;

	//int mixId;
};



//----------------------------------------------------------------
///	Simple dialog allowing the user to add a MIDI mapping to NPBMappingsContent's temporary array.
/*!
	\todo Label the various Components.
	\todo Remove the NPBMappingAdd-specific parts (plugin).
	\sa NPBMappingAdd
 */
class NPBMappingAddMIDI : public DialogWindow
{
  public:
	///	Constructor.
	/*!
		\param parent The NPBMappingsContent which can be considered the
		parent of this object.
		\param The colour of the background.
	 */
	NPBMappingAddMIDI(NPBMappingsContent *parent, NPBPlugin *plugin, Colour col);
	///	Destructor.
	~NPBMappingAddMIDI() {};
};

///	The contents of NPBMappingAddMIDI.
/*!
	\todo Remove all the unnecessary parts left over from NPBAddContent.
 */
class NPBAddMIDIContent : public Component,
							 ActionListener
{
  public:
	///	Constructor.
	/*!
		\param parent The NPBMappingsContent which can be considered the
		parent of this object.
		\param The colour of the background.
	 */
	NPBAddMIDIContent(NPBMappingsContent *parent, NPBPlugin *plugin, Colour col);
	///	Destructor.
	~NPBAddMIDIContent() {};

	///	Fills in the background with colour backColour.
	void paint(Graphics& g);
	///	This is used to draw a frame around the NiallsLabel Component.
	void paintOverChildren(Graphics& g);
	///	So we're notified when the user presses OK or Cancel.
	void actionListenerCallback(const String& m);

	///	To make sure the Components are all placed in the correct positions.
	void resized();
  private:
	//void setupComboBox(ComboBox *cb, int index);

	ComboBox *JoyNum;
	NiallsLabel *JoyControl;
	TextEditor *MinVal;
	TextEditor *MaxVal;
	ComboBox *PluginParameter;
	ComboBox *MIDINote;

	TextButton *OK;
	TextButton *Cancel;

	NPBMappingsContent *Parent;
	NPBPlugin *Plugin;

	///	Colour to paint the background.
	Colour backColour;

	//int mixId;
};

#endif
