//  NPBWindow.cpp - The main window, set up like a normal Windows
//				    window, with minimise, maximise and close
//				    buttons, and a resizer.
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#include "NPBWindow.h"
#include "NPBImages.h"
#include "NPBOptionsDialog.h"
#include "NPBAbout.h"
#include "NPBJoystickDialog.h"
#include "NPBVSTDialog.h"
#include "NPBMIDIDialog.h"
#include "NPBTutorialDialog.h"
#include "ComponentFactory.h"

static const int bevelWidth = 3;
static const int titleHeight = 28;

///---------------------------------------------------------------------------
NPBWindow::NPBWindow(String xmlPath, bool debug):
Component(T("Nialls  Pedal  Board")),
titleX(10),
titleY(21),
lookAndFeel(0)
{
	int i, tempint;
	String tempstr;
	String temppath;

	setParentThing(this);

	if(debug)
	{
		FileLogger *loggy = new FileLogger(File::getCurrentApplicationFile().getSiblingFile(T("NiallsPedalBoard.log")),
										   T("Niall's Pedal Board debug log"));
		Logger::setCurrentLogger(loggy);
	}

	factory = 0;
	Maximised = false;
	MoveWindow = false;
	OptionsOpen = false;
	MouseOffset_x = 0;
	MouseOffset_y = 0;
	JoystickSmoothing = false;
	PatchPath = String::empty;
	PluginPath = String::empty;
	abText1 = T("Start Audio");
	abText2 = T("Stop Audio");
	rbText1 = T("Record");
	rbText2 = T("Stop");

	//Necessary to avoid stupid assert errors in debug mode...
	//(the window's size is actually set by the createGUI() call below)
	setSize(1, 1);

	colour = Colours::lightblue;
	titleColour = Colours::darkgrey;

	Logger::writeToLog(T("NPBWindow::NPBWindow(): Finished setting up initial values."));

	//--Handle xml file/factory------
	if(xmlPath.isEmpty())
	{
		//where the xml file will be...
		temppath = SystemStats::getCurrentExecutableFileName();
#ifdef WIN32
		tempint = temppath.lastIndexOfChar('\\');
#else
		tempint = temppath.lastIndexOfChar('/');
#endif
		temppath = temppath.dropLastCharacters(temppath.length()-(tempint+1));
		temppath = temppath;
		temppath += T("NiallsPedalBoard.xml");
	}
	else
		temppath = xmlPath;

	tempstr = String::empty;
	tempstr << T("NPBWindow::NPBWindow(): temppath for ComponentFactory = ") << temppath;
	Logger::writeToLog(tempstr);

	factory = new ComponentFactory(this, temppath);

	Logger::writeToLog(T("NPBWindow::NPBWindow(): ComponentFactory has been created."));

	//--etc.------
	setOpaque(true);

	//--This stuff is broken, I think :(------
	//MemoryInputStream tempstream(NPBImages::safont_fnt, NPBImages::safont_fntSize, false);
	//FileInputStream tempstream(File("D://Program Files//Microsoft Visual Studio//MyProjects//Other Stuff//NiallsPedalBoard//images//test"));
	//SubvertAudioFont = new Typeface(tempstream);//I love this way of loading fonts :)
	//--------

	setVisible(true);
	addToDesktop(windowAppearsOnTaskbar|windowHasDropShadow);

	Logger::writeToLog(T("NPBWindow::NPBWindow(): Have added window to desktop."));

	//Can only set up joysticks after the window's been added to the desktop.
	Joysticks = enumerateDevices();
	HWND tempHwnd = static_cast<HWND>(getWindowHandle());
	for(i=0;i<Joysticks.size();i++)
		acquireJoystick(Joysticks[i]);

	Logger::writeToLog(T("NPBWindow::NPBWindow(): Have enumerated joysticks."));

	addAndMakeVisible(resizer = new ResizableCornerComponent(this));
	//resizer->setColours(colour.darker(0.2f), colour.darker(0.4f));

	if(debug)
		debugWriteSoundcardOptions();

	Soundcard = ASIOAudioIODevice::getASIODeviceNames()[0];

	tempstr = String::empty;
	tempstr << T("NPBWindow::NPBWindow(): Have set Soundcard to ") << Soundcard;
	Logger::writeToLog(tempstr);

	//--Now load previous settings (soundcard, joystick smoothing etc.)--

	//to get the application's directory (where the properties file will be)...
	tempstr = SystemStats::getCurrentExecutableFileName();
#ifdef WIN32
	tempint = tempstr.lastIndexOfChar('\\');
#else
	tempint = tempstr.lastIndexOfChar('/');
#endif
	tempstr = tempstr.dropLastCharacters(tempstr.length()-(tempint+1));
	tempstr = tempstr;

	tempstr += "NPBProperties.prp";
	File phil(tempstr);
	if(!phil.exists())
	{
		Logger::writeToLog(T("NPBWindow::NPBWindow(): Creating properties file."));
		phil.create(); //in case it doesn't exist yet
		phil.printf("PROP.   "); //without this, nothing will be saved in the file for some reason...
	}
	else
	{
		temppath = String::empty;
		temppath << T("NPBWindow::NPBWindow(): Properties path = ") << tempstr;
		Logger::writeToLog(temppath);
	}
	phil.setReadOnly(false);

	Logger::writeToLog(T("NPBWindow::NPBWindow(): Setting properties."));

	Prop = new PropertiesFile(phil, 0, false, false);
	//have to check this stuff, because the file may have just been created
	if(Prop->containsKey(T("JoystickSmoothing")))
	{
		JoystickSmoothing = Prop->getBoolValue(T("JoystickSmoothing"));
		setFilter(JoystickSmoothing);
	}
	if(Prop->containsKey(T("Soundcard")))
		Soundcard = Prop->getValue(T("Soundcard"));
	if(Prop->containsKey(T("Inputs")))
		Inputs.setBitRangeAsInt(0, 32, Prop->getIntValue(T("Inputs"), 0));
	if(Prop->containsKey(T("Outputs")))
		Outputs.setBitRangeAsInt(0, 32, Prop->getIntValue(T("Outputs"), 0));
	if(Prop->containsKey(T("PatchPath")))
		PatchPath = Prop->getValue(T("PatchPath"));
	if(Prop->containsKey(T("PluginPath")))
		PluginPath = Prop->getValue(T("PluginPath"));

	createGUI();

	Logger::writeToLog(T("NPBWindow::NPBWindow(): Have created the gui."));

	//Because the Organiser won't have been created until now.
	if(Prop->containsKey(T("EnvelopeDecrement")))
		setEnvelopeDecrement(static_cast<float>(Prop->getDoubleValue(T("EnvelopeDecrement"))));
	else
		setEnvelopeDecrement(0.0001f);
	if(Prop->containsKey(T("MIDIInputDevice")))
		setMIDIInputDevice(Prop->getIntValue(T("MIDIInputDevice")));
	if(Prop->containsKey(T("DirectMode")))
		setDirectMode(Prop->getBoolValue(T("DirectMode")));

	Logger::writeToLog(T("NPBWindow::NPBWindow(): Have set final properties."));

	Organiser->setIOs(Soundcard, Inputs, Outputs);

	Logger::writeToLog(T("NPBWindow::NPBWindow(): Have set Organiser's I/O's."));

	resized();

	Logger::writeToLog(T("NPBWindow::NPBWindow(): Have resized the gui."));
}

///---------------------------------------------------------------------------
NPBWindow::~NPBWindow()
{
	//Save settings (soundcard, joystick smoothing etc.)
	Prop->setValue(T("JoystickSmoothing"), JoystickSmoothing);
	Prop->setValue(T("Soundcard"), Soundcard);
	Prop->setValue(T("Inputs"), Inputs.getBitRangeAsInt(0, 32));
	Prop->setValue(T("Outputs"), Outputs.getBitRangeAsInt(0, 32));
	Prop->setValue(T("DirectMode"), getDirectMode());
	Prop->setValue(T("MIDIInputDevice"), getMIDIInputDevice());
	Prop->setValue(T("PatchPath"), PatchPath);
	Prop->setValue(T("PluginPath"), PluginPath);
	Prop->setValue(T("EnvelopeDecrement"), getEnvelopeDecrement());
	Prop->ensureWrittenToFile();
	delete Prop;

	if(factory)
		delete factory;

	//delete SubvertAudioFont;
    deleteAllChildren();
	JUCEApplication::quit();
}

///---------------------------------------------------------------------------
void NPBWindow::paint(Graphics& g)
{	
	int tempx, tempy;

	//Font tempfont(*SubvertAudioFont);
	Font tempfont(Typeface(T("Arial"), false, false));
	g.setFont(tempfont);

	g.setColour(Colour::greyLevel(0.4f));
    RectangleList visible;
    getVisibleArea(visible, false);
    visible.fill(g);

	GradientBrush gradientBrush(colour.brighter(0.2f), 0.0f, 0.0f,
                                colour, (float)getWidth(), (float)getHeight(),
                                true);
	g.setBrush (&gradientBrush);
	g.fillAll();

    g.setColour(titleColour);
    g.setFont(titleHeight * 0.6f, 1);

	if(titleX < 0)
		tempx = getWidth() + titleX;
	else
		tempx = titleX;
	if(titleY < 0)
		tempy = getHeight() + titleY;
	else
		tempy = titleY;
	g.drawSingleLineText(getName(), tempx, tempy);

	g.setColour (Colours::black.withAlpha (0.25f));
    g.drawRect (0, 0, getWidth(), getHeight());
	g.setColour (Colours::black.withAlpha (0.125f));
    g.drawRect (1, 1, (getWidth()-2), (getHeight()-2));
}

///---------------------------------------------------------------------------
void NPBWindow::closeButtonPressed()
{
    delete this;
}

///---------------------------------------------------------------------------
void NPBWindow::minButtonPressed()
{
	NativeDesktopWindow* const w = getNativeWindow();

	if(w->isMinimised())
		w->setMinimised(false);
	else
		w->setMinimised(true);
}

///---------------------------------------------------------------------------
void NPBWindow::maxButtonPressed()
{
	NativeDesktopWindow* const w = getNativeWindow();

	if(!Maximised)
	{
		oldSize = getBounds();
		Maximised = true;
		resizer->setVisible(false);
	}
	else
	{
		setBounds(oldSize);
		Maximised = false;
		resizer->setVisible(true);
	}

	w->setFullScreen(Maximised);
}

///---------------------------------------------------------------------------
void NPBWindow::AudioButtonPressed()
{
	if(AudioButton->getButtonText() == abText1)
	{
		if(Organiser->StartAudio(Soundcard, Inputs, Outputs))
			AudioButton->setButtonText(abText2);
	}
	else
	{
		Organiser->StopAudio();
		AudioButton->setButtonText(abText1);
	}
}

///---------------------------------------------------------------------------
void NPBWindow::OptionsButtonPressed()
{
	NPBOptionsDialog *dlg = new NPBOptionsDialog(colour);
	dlg->addToDesktop(windowHasDropShadow);

	NPBOptionsContent *cont = new NPBOptionsContent(this, colour);
	dlg->setContentComponent(cont);

	dlg->centreWithSize(400, 325);
	dlg->enterModalState();
}

///---------------------------------------------------------------------------
void NPBWindow::RecordButtonPressed()
{
	File filepath;
	String tempstr;

	if(AudioButton->getButtonText() != abText1)
	{
		if(RecordButton->getButtonText() == rbText1)
		{
			FileChooser dlg(T("Record to a wav..."), File(PatchPath), T("*.wav"));

			if(dlg.browseForFileToSave())
			{
				filepath = dlg.getResult();
				if(Organiser->startRecording(filepath))
					RecordButton->setButtonText(rbText2);
			}
		}
		else
		{
			Organiser->stopRecording();
			RecordButton->setButtonText(rbText1);
		}
	}
}

///---------------------------------------------------------------------------
void NPBWindow::actionListenerCallback(const String& m)
{
	int i;
	int tempint;
	OwnedArray<File> temparr;
	String tempstr;
	int menu_result;
	DialogWindow *dlg;

    if(closeButton->getName() == m)
        closeButtonPressed();
	else if(minButton->getName() == m)
        minButtonPressed();
	else if(maxButton->getName() == m)
        maxButtonPressed();
	else if(AudioButton->getName() == m)
        AudioButtonPressed();
	else if(OptionsButton->getName() == m)
		OptionsButtonPressed();
	else if(RecordButton->getName() == m)
		RecordButtonPressed();
	else if(FileMenu->getName() == m)
	{
		PopupMenu menu;

		menu.addItem(1, T("New"));
		menu.addSeparator();
		menu.addItem(2, T("Open"));
		menu.addItem(3, T("Save"));
		menu.addSeparator();
		menu.addItem(4, T("Quit"));

		menu_result = menu.showAt((getX()+FileMenu->getX()), (getY()+(FileMenu->getY()+FileMenu->getHeight())));
		switch(menu_result)
		{
			case 1: //new
				Organiser->clearOrganiser();
				break;
			case 2: //open
				Organiser->loadPatch();
				break;
			case 3: //save
				Organiser->savePatch();
				break;
			case 4: //quit
				delete this;
				break;
		}
	}
	else if(OptionsMenu->getName() == m)
	{
		PopupMenu menu;

		menu.addItem(1, T("Joystick Options"));
		menu.addItem(2, T("Audio Options"));
		menu.addItem(3, T("VST Options"));
		menu.addItem(4, T("MIDI Options"));

		menu_result = menu.showAt((getX()+OptionsMenu->getX()), (getY()+(OptionsMenu->getY()+OptionsMenu->getHeight())));
		switch(menu_result)
		{
			case 1:
				dlg = new NPBJoystickDialog(this, colour);
				dlg->addToDesktop(windowHasDropShadow);

				dlg->centreWithSize(400, 300);
				dlg->enterModalState();
				break;
			case 3:
				dlg = new NPBVSTDialog(this, colour);
				dlg->addToDesktop(windowHasDropShadow);

				dlg->centreWithSize(400, 300);
				dlg->enterModalState();
				break;
			case 4:
				dlg = new NPBMIDIDialog(this, colour);
				dlg->addToDesktop(windowHasDropShadow);

				dlg->centreWithSize(400, 300);
				dlg->enterModalState();
				break;
			case 2:
				dlg = new NPBOptionsDialog(colour);
				dlg->addToDesktop(windowHasDropShadow);

				NPBOptionsContent *cont = new NPBOptionsContent(this, colour);
				dlg->setContentComponent(cont);

				dlg->centreWithSize(400, 300);
				dlg->enterModalState();
				break;
		}
	}
	else if(HelpMenu->getName() == m)
	{
		PopupMenu menu;

		File tempfile = File::getCurrentApplicationFile().getParentDirectory().getChildFile(T("docs"));
		tempint = tempfile.findChildFiles(temparr, false, false, T("*.txt"));
		for(i=0;i<tempint;i++)
		{
			tempstr = temparr[i]->getFileNameWithoutExtension();
			menu.addItem((i+1), tempstr);
		}

		menu.addSeparator();
		menu.addItem((i+1), T("About"));

		menu_result = menu.showAt((getX()+HelpMenu->getX()), (getY()+(HelpMenu->getY()+HelpMenu->getHeight())));

		
		if(menu_result < (i+1))
		{
			tempstr = temparr[(menu_result-1)]->getFullPathName();
			dlg = new NPBTutorialDialog(colour, tempstr);
			dlg->addToDesktop(windowHasDropShadow);

			dlg->centreWithSize(450, 600);
			dlg->enterModalState();
		}
		if(menu_result == (i+1))
		{
			dlg = new NPBAbout(colour);
			dlg->addToDesktop(windowHasDropShadow);

			dlg->centreWithSize(300, 280);
			dlg->enterModalState();
		}
	}
}

///---------------------------------------------------------------------------
void NPBWindow::joystickCallback(JoystickEvent& joystate, int index)
{
	Organiser->joystickCallback(joystate, index);
}

///---------------------------------------------------------------------------
void NPBWindow::resized()
{
	factory->resizeComponents();

	resizer->setBounds((getWidth()-16), (getHeight()-16), 16, 16);
}

///---------------------------------------------------------------------------
void NPBWindow::mouseDown(const MouseEvent &e)
{
	if(!Maximised)
	{
		if(e.getMouseDownY() < titleHeight)
			MoveWindow = true;
	}

	MouseOffset_x = e.getScreenX();
	MouseOffset_y = e.getScreenY();
}

///---------------------------------------------------------------------------
void NPBWindow::mouseDrag(const MouseEvent &e)
{
	int tempx, tempy;

    int newX = getX();
    int newY = getY();
    int newW = getWidth();
    int newH = getHeight();

    const Rectangle screen (Desktop::getInstance().getAllMonitorDisplayAreas()
                                                  .getBounds());

	tempx = e.getScreenX() - MouseOffset_x;
	tempy = e.getScreenY() - MouseOffset_y;

	if(MoveWindow)
	{

		tempx += getScreenX();
		tempy += getScreenY();

		MouseOffset_x = e.getScreenX();
		MouseOffset_y = e.getScreenY();

		setTopLeftPosition(tempx, tempy);
	}
}

///---------------------------------------------------------------------------
void NPBWindow::mouseUp(const MouseEvent &e)
{
	if(MoveWindow)
		MoveWindow = false;
}

///---------------------------------------------------------------------------
void NPBWindow::DialogOK(String soundcard, BitArray input, BitArray output)
{
	Soundcard = soundcard;
	Inputs = input;
	Outputs = output;

	Organiser->setIOs(soundcard, input, output);
}

///---------------------------------------------------------------------------
void NPBWindow::DialogCancel()
{
}

///---------------------------------------------------------------------------
void NPBWindow::setEnvelopeDecrement(float val)
{
	if(Organiser)
		Organiser->setEnvelopeDecrement(val);
}

///---------------------------------------------------------------------------
float NPBWindow::getEnvelopeDecrement()
{
	if(Organiser)
		return Organiser->getEnvelopeDecrement();
	else
		return 0.0001f;
}

///---------------------------------------------------------------------------
void NPBWindow::setBpm(float val)
{
	if(Organiser)
		Organiser->setBpm(val);
}

///---------------------------------------------------------------------------
float NPBWindow::getBpm()
{
	if(Organiser)
		return Organiser->getBpm();
	else
		return 120.0f;
}

///---------------------------------------------------------------------------
bool NPBWindow::getDirectMode()
{
	if(Organiser)
		return Organiser->getDirectMode();
	else
		return false;
}

///---------------------------------------------------------------------------
void NPBWindow::setDirectMode(bool val)
{
	if(Organiser)
		Organiser->setDirectMode(val);
}

///---------------------------------------------------------------------------
void NPBWindow::setResizer(bool val)
{
	resizer->setVisible(val);
}

///---------------------------------------------------------------------------
void NPBWindow::setMaximise(bool val)
{
	NativeDesktopWindow* const w = getNativeWindow();

	if(val)
	{
		oldSize = getBounds();
		resizer->setVisible(false);
	}
	else
	{
		setBounds(oldSize);
		resizer->setVisible(true);
	}
	w->setFullScreen(val);
}

///---------------------------------------------------------------------------
void NPBWindow::setColourRed(int val)
{
	Colour *tempcol;

	tempcol = new Colour(val, colour.getGreen(), colour.getBlue(), (uint8)255);
	colour = *tempcol;

	//resizer->setColours(colour.darker(0.2f), colour.darker(0.4f));

	delete tempcol;
}

///---------------------------------------------------------------------------
void NPBWindow::setColourGreen(int val)
{
	Colour *tempcol;

	tempcol = new Colour(colour.getRed(), val, colour.getBlue(), (uint8)255);
	colour = *tempcol;

	//resizer->setColours(colour.darker(0.2f), colour.darker(0.4f));

	delete tempcol;
}

///---------------------------------------------------------------------------
void NPBWindow::setColourBlue(int val)
{
	Colour *tempcol;

	tempcol = new Colour(colour.getRed(), colour.getGreen(), val, (uint8)255);
	colour = *tempcol;

	//resizer->setColours(colour.darker(0.2f), colour.darker(0.4f));

	delete tempcol;
}

///---------------------------------------------------------------------------
void NPBWindow::setTitleColourRed(int val)
{
	Colour *tempcol;

	tempcol = new Colour(val, titleColour.getGreen(), titleColour.getBlue(), (uint8)255);
	titleColour = *tempcol;

	delete tempcol;
}

///---------------------------------------------------------------------------
void NPBWindow::setTitleColourGreen(int val)
{
	Colour *tempcol;

	tempcol = new Colour(titleColour.getRed(), val, titleColour.getBlue(), (uint8)255);
	titleColour = *tempcol;

	delete tempcol;
}

///---------------------------------------------------------------------------
void NPBWindow::setTitleColourBlue(int val)
{
	Colour *tempcol;

	tempcol = new Colour(titleColour.getRed(), titleColour.getGreen(), val, (uint8)255);
	titleColour = *tempcol;

	delete tempcol;
}

///---------------------------------------------------------------------------
void NPBWindow::setAudioButtonText(String val1, String val2)
{
	abText1 = val1;
	abText2 = val2;
}

///---------------------------------------------------------------------------
void NPBWindow::setRecordButtonText(String val1, String val2)
{
	rbText1 = val1;
	rbText2 = val2;
}

///---------------------------------------------------------------------------
void NPBWindow::setMIDIInputDevice(int index)
{
	if(Organiser)
		Organiser->setMIDIInputDevice(index);
}

///---------------------------------------------------------------------------
int NPBWindow::getMIDIInputDevice()
{
	if(Organiser)
		return Organiser->getMIDIInputDevice();
	else
		return -1;
}

///---------------------------------------------------------------------------
void NPBWindow::debugWriteSoundcardOptions()
{
	long i, j;
	int tempint;
	String tempstr;
	ASIOAudioIODevice *tempAudio;
	StringArray chanArray;

	StringArray nameArray = ASIOAudioIODevice::getASIODeviceNames();

	for(i=0;i<nameArray.size();i++)
	{
		tempstr = T("Soundcard Device[");
		tempstr << i << T("] = ") << nameArray[i];
		Logger::writeToLog(tempstr);

		tempAudio = new ASIOAudioIODevice(nameArray[i],
										  static_cast<HWND>(getNativeWindow()->getNativeHandle()),
										  false);
		chanArray = tempAudio->getOutputChannelNames();
		for(j=0;j<chanArray.size();j++)
		{
			tempstr = T("Output Channel [");
			tempstr << j << T("] = ") << chanArray[j];
			Logger::writeToLog(tempstr);
		}
		chanArray = tempAudio->getInputChannelNames();
		for(j=0;j<chanArray.size();j++)
		{
			tempstr = T("Input Channel [");
			tempstr << j << T("] = ") << chanArray[j];
			Logger::writeToLog(tempstr);
		}
		tempint = tempAudio->getNumSampleRates();
		for(j=0;j<tempint;j++)
		{
			tempstr = T("Samplerate [");
			tempstr << j << T("] = ") << tempAudio->getSampleRate(j);
			Logger::writeToLog(tempstr);
		}
		tempint = tempAudio->getNumBufferSizesAvailable();
		for(j=0;j<tempint;j++)
		{
			tempstr = T("BufferSize[");
			tempstr << j << T("] = ") << tempAudio->getBufferSizeSamples(j);
			Logger::writeToLog(tempstr);
		}
		delete tempAudio;
	}
}
